@@ -82,7 +82,17 @@ push @patch, (
 	      qr/^5\.14\.[0123]$/ ], # to be fixed in 5.14.4
     subs => [ [ \&_patch_join_exact] ],
   },
-
+  {
+    perl => [ qr/^5\.17\.[78]$/ ],
+    # broken in 5.17.8, 2.006-2.007
+    subs => [ [ \&_patch_socket_inet_ntop ] ],
+  },
+  {
+    perl => [ qr/^5\.14\.[0123]$/,
+              qr/^5\.15\.[012]$/ ],
+    # regression in 5.14, fixed in v5.15.3-232-g1bac5ec
+    subs => [ [ \&_patch_anonymise_cv_maybe ] ],
+  },
 );
 
 sub _add_patchlevel {
@@ -104,6 +114,7 @@ sub _add_patchlevel {
   }
   close $in;
   close $out;
+  print STDERR "patched: $line\n";
   return $success;
 }
 
@@ -236,7 +247,6 @@ END
   _add_patchlevel($vers, "RT#111594 Socket::unpack_sockaddr_un heap-buffer-overflow");
 }
 
-
 sub _patch_eval_start
 {
   _patch(<<'END');
@@ -340,5 +350,77 @@ END
   _add_patchlevel(@_, "RT#115994 S_join_exact global-buffer-overflow");
 }
 
+sub _patch_socket_inet_ntop
+{
+  my $vers = shift;
+  my $patch = <<'END';
+--- cpan/Socket/Socket.xs
++++ cpan/Socket/Socket.xs
+@@ -934,8 +934,13 @@ inet_ntop(af, ip_address_sv)
+ #endif
+ 		      "Socket::inet_ntop", af);
+ 	}
+-
+-	Copy(ip_address, &addr, sizeof addr, char);
++	if (addrlen < sizeof(addr)) {
++	   Copy(ip_address, &addr, addrlen, char);
++           Zero(&addr+addrlen, sizeof(addr)-addrlen, char);
++	}
++	else {
++	  Copy(ip_address, &addr, sizeof addr, char);
++	}
+ 	inet_ntop(af, &addr, str, sizeof str);
+ 
+ 	ST(0) = sv_2mortal(newSVpvn(str, strlen(str)));
+END
+
+  #; )
+  _patch($patch);
+  _add_patchlevel($vers, "RT#82119 Socket::inet_ntop heap-buffer-overflow");
+}
+
+sub _patch_anonymise_cv_maybe
+{
+  my $vers = shift;
+  my $patch = <<'END';
+commit 1bac5ecc108e6bb05752e5aef66c6890163aff39
+Author: Brian Fraser <fraserbn@gmail.com>
+Date:   Mon Sep 26 13:48:52 2011 -0700
+
+    sv.c: S_anonymise_cv_maybe UTF8 cleanup.
+
+diff --git a/sv.c b/sv.c
+index d71f901..a3a2c74 100644
+--- sv.c.orig
++++ sv.c
+@@ -5893,7 +5893,6 @@ Perl_sv_replace(pTHX_ register SV *const sv, register SV *const nsv)
+ STATIC void
+ S_anonymise_cv_maybe(pTHX_ GV *gv, CV* cv)
+ {
+-    char *stash;
+     SV *gvname;
+     GV *anongv;
+ 
+@@ -5913,10 +5912,10 @@ S_anonymise_cv_maybe(pTHX_ GV *gv, CV* cv)
+     }
+ 
+     /* if not, anonymise: */
+-    stash  = GvSTASH(gv) && HvNAME(GvSTASH(gv))
+-              ? HvENAME(GvSTASH(gv)) : NULL;
+-    gvname = Perl_newSVpvf(aTHX_ "%s::__ANON__",
+-					stash ? stash : "__ANON__");
++    gvname = (GvSTASH(gv) && HvNAME(GvSTASH(gv)) && HvENAME(GvSTASH(gv)))
++                    ? newSVhek(HvENAME_HEK(GvSTASH(gv)))
++                    : newSVpvn_flags( "__ANON__", 8, 0 );
++    sv_catpvs(gvname, "::__ANON__");
+     anongv = gv_fetchsv(gvname, GV_ADDMULTI, SVt_PVCV);
+     SvREFCNT_dec(gvname);
+ 
+END
+
+  #; )
+  _patch($patch);
+  _add_patchlevel($vers, "RT#91678 S_anonymise_cv_maybe UTF8 cleanup");
+}
 
 1;
